#!/usr/bin/perl

# Dovecot mbox to maildir convert for all users
# See: 
# 
# https://wiki.dovecot.org/Migration/MailFormat
# https://gist.github.com/nyergler/1709069
# https://github.com/bluebird75/maildir2mbox/blob/master/maildir2mbox.py

use Unix::PasswdFile;
use Getopt::Long;

# Debugging switch (0|1|2):
# 0 = off
# 1 = log to syslog
# 2 = log to screen
#
$DEBUG = "2";
if ($DEBUG) {
    if ($DEBUG eq "1") {
        use Sys::Syslog qw( :DEFAULT setlogsock);
    }
}

$dovecot_conf = '/etc/dovecot/conf.d/10-mail.conf';
$procmail_conf = '/etc/procmailrc';

$python_three = `cat /etc/build |grep 5210R|wc -l`;
chomp($python_three);

if ($python_three eq '1') {
    $py_convert = '/usr/sausalito/bin/maildir2mbox_python3.py';
}
else {
    $py_convert = '/usr/sausalito/bin/maildir2mbox_python2.py'
}

# Show header:
&header;

#
### Check if we are 'root':
#
&root_check;

GetOptions(
  'mbox'        => sub { Mbox() },
  'maildir'     => sub { Maildir() },
  'help|h|?'    => sub { help() },
) or &help();

# Sense check:
if ($dodad eq '') {
    &help("Please specify some command line options.");
}

exit(0);

#
### Subs:
#

sub check {
    $is_mbox = `cat /etc/procmailrc |grep "HOME/mbox"|wc -l`;
    chomp($is_mbox);
    $is_maildir = `cat /etc/procmailrc |grep "HOME/Maildir"|wc -l`;
    chomp($is_maildir);
}

sub Mbox {
    $dodad = '1';
    &check();
    if ($is_mbox eq '1') {
        &help("Mailboxes are already in 'mbox' format!");
    }

    # Fix Dovecot configuration:
    print "sed -i 's|^mail_location = .*|mail_location = mbox:~/mail/:INBOX=mbox|g' $dovecot_conf\n";
    system("sed -i 's|^mail_location = .*|mail_location = mbox:~/mail/:INBOX=mbox|g' $dovecot_conf");

    # Fix procmailrc:
    print "sed -i 's|^ORGMAIL=.*|ORGMAIL=\$HOME/mbox|g' $procmail_conf\n";
    system("sed -i 's|^ORGMAIL=.*|ORGMAIL=\$HOME/mbox|g' $procmail_conf");

    # Restart Dovecot:
    system("systemctl restart dovecot");

    # Convert mailboxes of all users:
    $pw = new Unix::PasswdFile "/etc/passwd";
    foreach $user ($pw->users) {

        undef $home;
        $home = $pw->home($user);

        $user_Maildir = $home . '/Maildir';
        $user_Mbox = $home . '/mbox';

        $uid = $pw->uid($user);
        $gid = $pw->gid($user);

        if (-d $user_Maildir) {
            print "$py_convert $user_Maildir $user_Mbox\n";
            system("$py_convert $user_Maildir $user_Mbox");
        }
    }
    exit(0);
}

sub Maildir {
    $dodad = '1';
    &check();
    if ($is_maildir eq '1') {
        &help("Mailboxes are already in 'maildir' format!");
    }

    # Fix Dovecot configuration:
    print "sed -i 's|^mail_location = .*|mail_location = maildir:~/Maildir:LAYOUT=fs|g' $dovecot_conf\n";
    system("sed -i 's|^mail_location = .*|mail_location = maildir:~/Maildir:LAYOUT=fs|g' $dovecot_conf");

    # Fix procmailrc:
    print "sed -i 's|^ORGMAIL=.*|ORGMAIL=\$HOME/Maildir/|g' $procmail_conf\n";
    system("sed -i 's|^ORGMAIL=.*|ORGMAIL=\$HOME/Maildir/|g' $procmail_conf");

    # Restart Dovecot:
    system("systemctl restart dovecot");

    # Convert mailboxes of all users:
    $pw = new Unix::PasswdFile "/etc/passwd";
    foreach $user ($pw->users) {
        undef $home;
        $home = $pw->home($user);

        $user_Maildir = $home . '/Maildir';
        $user_Mbox = $home . '/mbox';

        $uid = $pw->uid($user);
        $gid = $pw->gid($user);

        if (-f $user_Mbox) {
            print "$user: dsync -u $user mirror mbox:~/mail:INBOX=~/mbox\n";
            system("dsync -u $user mirror mbox:~/mail:INBOX=~/mbox");
        }
    }
    exit(0);
}

sub header {
    print "┌──────────────────────────────────────────────────────────────┐\n";
    print "│ BlueOnyx Mbox/Maildir & Maildir/Mbox converter               │\n";
    print "└──────────────────────────────────────────────────────────────┘\n\n";
    $header_sent++;
}

sub help {
    $error = shift || "";
    if ($header_sent eq '0') {
        &header;
    }
    if ($error) {
        print "ERROR: $error\n";
    }
    print "\n";
    print "Example: $0 --mbox\n";
    print "Example: $0 --maildir\n";
    print "\n";
    print "         --mbox     Converts all mailboxes to mbox format\n";
    print "         --maildir  Converts all mailboxes to maildir format\n";
    print "         -h|--help  This help text\n\n";
    exit(1);
}

sub root_check {
    my $id = `id -u`;
    chomp($id);
    if ($id ne "0") {
        &help("$0 must be run by user 'root'!");
    }
}

sub debug_msg {
    if ($DEBUG eq "1") {
        $msg = shift;
        setlogsock('unix');
        openlog($0,'','user');
        syslog('info', "$ARGV[0]: $msg");
        closelog;
    }
    if ($DEBUG eq "2") {
        my $msg = shift;
        print $msg;
    }
}

# 
# Copyright (c) 2008-2019 Michael Stauber, SOLARSPEED.NET
# Copyright (c) 2008-2019 Team BlueOnyx, BLUEONYX.IT
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#     notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#     notice, this list of conditions and the following disclaimer in 
#     the documentation and/or other materials provided with the 
#     distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#     contributors may be used to endorse or promote products derived 
#     from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
# You acknowledge that this software is not designed or intended for 
# use in the design, construction, operation or maintenance of any 
# nuclear facility.
# 
